/*********************************************************************
 * 360 degree true motion in Atmosphere script. 
 *  
 * http://superted.freeshell.org/atmosphere/tutorials/scripts/truemotion
 * 
 * This will allow your actor in a world to have true motion 
 * in Atmo via a combination of 
 *  - mouse navigation (UP/DOWN/LEFT/RIGHT)
 *  - actor.targetVelocity ('r' / 'f') on the keyboard, in feet per second
 *
 * This scripts is useful in worlds where you might require full motion. 
 * Default Atmosphere navigation restricts your motion to the X/Y plane, 
 * and you have trouble moving with changing Z coordinates. You can 
 * examine this by moving your avatar to face directly upwards 
 * in a world (90 degrees UP), then trying to move left or right. You may
 * notice that your avatar spins around as if on a turnstile. (Please note
 * i've just pulled 50% of this explanation out of my ass). 
 * 
 * Depending on your point of view, this can either be a good thing or
 * a bad thing. If your expectation is for 2D movement, this probably
 * a good thing as your avatar wont get disorienatated. But if your 
 * expectation is for 3D movement as in flight simulations, this is
 * counter intuitive. Thus, this script is good for: 
 * 
 *  - flight simulations
 *  - underwater swimming simulations
 *  - space simulations
 *  - Eschers Relativity simulation
 *  etc.
 *********************************************************************/


// Setup -------------------------------

// Press this key to accelerate. 
// Default: 82 == 'r' key
this.accelerateKey = 82;

// Default: 70 == 'f' key
this.decelerateKey = 70;

// Maximum velocity in feet per second
this.topspeed = 100;

// -------------------------------------

// Little intro.... 
chat.print('Use your mouse to navigate');
chat.print('R increase velocity by 1 foot/second');
chat.print('F decrease velocity by 1 foot/second');


/******************************************************
 * Stars background
 ******************************************************/
this.backg = Model("backg.aer").add();
this.backg.timestep = function() {
	this.position = theActor.position;
}
addAnimator(this.backg);

/******************************************************
 * Navigation Box
 ******************************************************/
// Navigation box... this is how we eliminate the 
// Y navigational problem
navanch = Array(4);
navbox = undefined;

// Sets up the navigation box. We acheive perfect 360 free motion
// in atmosphere via a little bit of cheating :o
// Within the navitation box, there are predefined anchors 
// which let us know relative to where the navigation
// box is located, where the NEXT rotation should be. 
// For example, we dont calculate when an actor moves to the 
// left, what the orientation of the actor should be - 
// we just take it from the navigation box and adjust the
// anchor to world coordinates.  It works, trust me! :)

this.setupNavBox = function() {	
	navbox = SceneGroup('./navobj.atmo').add();
	navbox.onLoad = function() {
		// Hide the box.... 
		this.getSolidObject(0).opacity = 0;
		
		// This anchor represents the next UP motion
		navanch[0] = this.getChild('forwardcenterup');	
		// This anchor represents the next DOWN motion
		navanch[1] = this.getChild('forwardcenterdown');
		// This anchor represents the next LEFT motion
		navanch[2] = this.getChild('forwardleftcenter');	
		// This anchor represents the next RIGHT motion
		navanch[3] = this.getChild('forwardrightcenter');
		// This anchor represents the next FORWARD motion
		// We use this mainly to calculate the 
		// actors target velocity
		navanch[4] = this.getChild('forwardcentercenter');
		
		navbox.onLoad = undefined;
		removeAnimator(navbox);
	}
	addAnimator(navbox);
}
this.setupNavBox();

/******************************************************
 * Camera / Actor movement and orientation
 ******************************************************/
this.cam = application.getView(0).getCamera(0);
this.cam.view = application.getView(0);



// Sets up the SPEED odometer for the user... 
// We make it as a variable of the camera, since there are both 
// interlinked. 
this.cam.max = this.topspeed;
this.cam.speed = 0;
this.cam.txt = stageModel.getViewpointObject("speed.mtx");
this.cam.txt.cam = this.cam;
this.cam.txt.onLoad = function() {
	this.onLoad =undefined;
	removeAnimator(this);

	this.cam.txt = this.scene.getInstance("ins").getInstance2d();	
} 
addAnimator(this.cam.txt);
this.cam.setSpeed = function(spd) {
	if(spd <= this.max && spd >=0) {
		this.speed = spd;
		this.txt.text = spd+"";
	}
}

// Stores the actors latest rotation. 
this.cam.rot = undefined;
this.cam.timestep = function() {

	// Kills all the actors default motion
	theActor.keyLeftRightMovesEnabled  = false;
	theActor.keyUpDownMovesEnabled  = false;
	theActor.keyForwardBackwardMovesEnabled  = false;
	theActor.keyLeftRightTurnsEnabled = false;
	theActor.ignoreCollidePreference = true;
	theActor.forcedCollideSetting = true;
	theActor.ignoreGravityPreference = true;
	theActor.forcedGravitySetting = false;
	theActor.showAvatar = false;
	application.showAvatar = false;
	
	// The center of the screen
	var centerX = this.view.imageWidth/2.0;
	var centerY = this.view.imageHeight/2.0;

	// Where the mouse is currently on the screen
	var xdiff = this.view.mouseX - centerX;
	var ydiff = this.view.mouseY - centerY;

	// We create a vector for the mouse location and normalise it. 
	// This will give us values [0..1] by which we "move" the orientation
	// of the actor to. 
	var norm  = Vector(xdiff, ydiff, centerX).normalized;
	
	// The initial orientation when the navigation box has loaded.
	if(this.oldrot==undefined && navanch[4]!=undefined) {
		this.oldrot = this.orientation;
	}

	// Do the mouse movement. We get 2 rotations out of this: 
	// Up/Down    : when the normalised vector Y is Positive/Negative
	// Right/Left : when the normalised vector X is Positive/Negative
	if(navanch[3]!=undefined) {
		if(norm.y < 0) {
			// Rot down
			this.rot = navbox.mapLocalTransformToWorldTransform(navanch[0].transform).rotation;
		} else {
			// Rot up
			this.rot = navbox.mapLocalTransformToWorldTransform(navanch[1].transform).rotation;
		}
		if(norm.x < 0) {
			// Rot left
			this.rot2 = navbox.mapLocalTransformToWorldTransform(navanch[2].transform).rotation;
		} else {
			// Rot right.
			this.rot2 = navbox.mapLocalTransformToWorldTransform(navanch[3].transform).rotation;
		}
	}
	
	// If we have rotations to use, we use it for the actor. 
	if(this.rot!=undefined) {
		
		// Blend the 2 Up/Down and Right/Left rotations together. 
		// For the actor. Since the actor is attached to the camera, 
		// rotating the camera will rotate the actor too.
		this.orientation = this.oldrot.blend(this.rot,Math.abs(norm.y));
		this.orientation = this.orientation.blend(this.rot2, Math.abs(norm.x));
		
		// Maintain the previous actor rotation incase we lose it somehow... ? 
		this.oldrot = this.orientation;
		
		// Do the same as we did for the actor/camera to the navigation box
		// So each timestep we start from the same position as the actor
		navbox.orientation = navbox.orientation.blend(this.rot,Math.abs(norm.y));
		navbox.orientation = navbox.orientation.blend(this.rot2,Math.abs(norm.x));
	}
	
	// Makes the thrusters go forward!
	// Note: there is no reverse ;) 
	// The Actor velocity is a combination of the speed set by the user 
	// by typing the keys 'r' or 'f' and the 'forwardcentrecenter' anchor
	// located in the navigation box. 
	if(navbox!=undefined && navanch[4]!=undefined) {
		actor.targetVelocity = navbox.mapLocalPositionToWorldPosition(navanch[4].position).negated.scale(this.speed*2.0);
	}
	
}
addAnimator(this.cam);


/******************************************************************
 * Key event handler
 * 
 * Handles the throttle +/-:
 * 'r' == accelerate (default)
 * 'f' == decelerate (default)
 ******************************************************************/ 
this.cam.keyb = KeyEventHandler();
this.cam.keyb.setFilterOnKeyDown();
this.cam.keyb.cam = this.cam;
this.cam.keyb.accelerateKey = this.accelerateKey
this.cam.keyb.decelerateKey = this.decelerateKey;
this.cam.keyb.onEvent = function(event) {
	switch(event.keyCode) {
		case this.accelerateKey: {
			// Increase velocity
			this.cam.setSpeed(this.cam.speed+1);break;
		} case this.decelerateKey: {
			// Decrease velocity
			this.cam.setSpeed(this.cam.speed-1);break;
		}

	}
}

